#!/usr/bin/python3
import sys, os, pwd, grp, signal, time, base64
from resource_management import *
from resource_management.core.exceptions import Fail
from resource_management.core.logger import Logger
from resource_management.core.resources.system import Execute, Directory, File
from resource_management.core.shell import call
from resource_management.core.system import System
from resource_management.libraries.functions.default import default

def check_airflow_installed():
	import os
	return os.path.exists("/usr/local/bin/airflow")

def airflow_make_systemd_scripts_webserver(env):
	import params
	env.set_params(params)

	confFileText = format("""[Unit]
Description=Airflow Web Server
After=network.target

[Service]
User={airflow_user}
Group={airflow_group}
Environment="AIRFLOW_HOME={airflow_home}"
Environment="AIRFLOW_CONFIG={airflow_home}/airflow.cfg"
ExecStart=/bin/bash -c 'airflow webserver --port {webserver_port} --pid {airflow_webserver_pid_file}'
Restart=always

[Install]
WantedBy=multi-user.target

""")

	with open("/usr/lib/systemd/system/airflow-webserver.service", 'w') as configFile:
		configFile.write(confFileText)
	configFile.close()

	
	confFileText = format("AIRFLOW_HOME={airflow_home}")

	if os.path.exists("/etc/sysconfig"):
		conf_file_path = "/etc/sysconfig/airflow"
	else:
		conf_file_path = "/etc/default/airflow"
	
	with open(conf_file_path, 'w') as configFile:
		configFile.write(confFileText)

	Execute("systemctl daemon-reload")

def airflow_make_systemd_scripts_scheduler(env):
	import params
	env.set_params(params)

	confFileText = format("""[Unit]
Description=Airflow Scheduler
After=network.target

[Service]
User={airflow_user}
Group={airflow_group}
Environment="AIRFLOW_HOME={airflow_home}"
Environment="AIRFLOW_CONFIG={airflow_home}/airflow.cfg"
ExecStart=/bin/bash -c 'airflow scheduler --pid {airflow_scheduler_pid_file}'
Restart=always

[Install]
WantedBy=multi-user.target

""")

	with open("/usr/lib/systemd/system/airflow-scheduler.service", 'w') as configFile:
		configFile.write(confFileText)
	configFile.close()

	confFileText = format("AIRFLOW_HOME={airflow_home}")

	if os.path.exists("/etc/sysconfig"):
		conf_file_path = "/etc/sysconfig/airflow"
	else:
		conf_file_path = "/etc/default/airflow"
	
	with open(conf_file_path, 'w') as configFile:
		configFile.write(confFileText)

	Execute("systemctl daemon-reload")

def airflow_make_systemd_scripts_worker(env):
	import params
	env.set_params(params)

	confFileText = format("""[Unit]
Description=Airflow Celery Worker
After=network.target

[Service]
User={airflow_user}
Group={airflow_group}
Environment="AIRFLOW_HOME={airflow_home}"
Environment="AIRFLOW_CONFIG={airflow_home}/airflow.cfg"
ExecStart=/bin/bash -c 'airflow celery worker --pid {airflow_worker_pid_file}'
Restart=always

[Install]
WantedBy=multi-user.target

""")

	with open("/usr/lib/systemd/system/airflow-worker.service", 'w') as configFile:
		configFile.write(confFileText)
	configFile.close()

	confFileText = format("AIRFLOW_HOME={airflow_home}")

	if os.path.exists("/etc/sysconfig"):
		conf_file_path = "/etc/sysconfig/airflow"
	else:
		conf_file_path = "/etc/default/airflow"
	
	with open(conf_file_path, 'w') as configFile:
		configFile.write(confFileText)

	Execute("systemctl daemon-reload")

def airflow_make_systemd_scripts_flower(env):
	import params
	env.set_params(params)

	confFileText = format("""[Unit]
Description=Airflow Celery Flower
After=network.target

[Service]
User={airflow_user}
Group={airflow_group}
Environment="AIRFLOW_HOME={airflow_home}"
Environment="AIRFLOW_CONFIG={airflow_home}/airflow.cfg"
ExecStart=/bin/bash -c 'airflow celery flower'
Restart=always

[Install]
WantedBy=multi-user.target

""")

	with open("/usr/lib/systemd/system/airflow-flower.service", 'w') as configFile:
		configFile.write(confFileText)
	configFile.close()

	confFileText = format("AIRFLOW_HOME={airflow_home}")

	if os.path.exists("/etc/sysconfig"):
		conf_file_path = "/etc/sysconfig/airflow"
	else:
		conf_file_path = "/etc/default/airflow"
	
	with open(conf_file_path, 'w') as configFile:
		configFile.write(confFileText)

	Execute("systemctl daemon-reload")


def airflow_make_systemd_scripts_kerberos(env):
	import params
	env.set_params(params)

	confFileText = format("""[Unit]
Description=Airflow Kerberos
After=network.target

[Service]
User={airflow_user}
Group={airflow_group}
Environment="AIRFLOW_HOME={airflow_home}"
Environment="AIRFLOW_CONFIG={airflow_home}/airflow.cfg"
ExecStart=/bin/bash -c 'airflow kerberos'
Restart=always

[Install]
WantedBy=multi-user.target

""")

	with open("/usr/lib/systemd/system/airflow-kerberos.service", 'w') as configFile:
		configFile.write(confFileText)
	configFile.close()

	confFileText = format("AIRFLOW_HOME={airflow_home}")

	if os.path.exists("/etc/sysconfig"):
		conf_file_path = "/etc/sysconfig/airflow"
	else:
		conf_file_path = "/etc/default/airflow"
	
	with open(conf_file_path, 'w') as configFile:
		configFile.write(confFileText)

	Execute("systemctl daemon-reload")


def airflow_make_startup_script(env):
	import params
	env.set_params(params)

	confFileText = format("""#!/bin/bash

	export AIRFLOW_HOME={airflow_home} && $(which airflow) $1 --pid {airflow_home}/airflow-sys-$1.pid
	""")

	with open(format("{airflow_home}/airflow_control.sh"), 'w') as configFile:
		configFile.write(confFileText)
	configFile.close()
	Execute(format("chmod 755 {airflow_home}/airflow_control.sh"))

def airflow_generate_config_for_section(sections):
	"""
	Generating values for airflow.cfg for each section.
	This allows to add custom-site configuration from ambari to cfg file.
	"""
	result = {}
	for section, data in sections.items():
		section_config = ""
		for key, value in data.items():
			section_config += format("{key} = {value}\n")
		result[section] = section_config
	return result

def airflow_configure(env):
	import params
	env.set_params(params)

# SSL configuration
	airflow_webserver_site_config = params.config['configurations']['airflow-webserver-site']
	webserver_enable_ssl_value = airflow_webserver_site_config.get('enable_ssl', '')
	webserver_hostname_value = airflow_webserver_site_config.get('web_server_host', '')
	webserver_port_value = airflow_webserver_site_config.get('web_server_port', '')

	airflow_webserver_site_config_copy = params.config['configurations']['airflow-webserver-site'].copy()

	if webserver_enable_ssl_value == 'False':
		airflow_webserver_site_config_copy['base_url'] = "http://{0}:{1}".format(webserver_hostname_value,webserver_port_value)
	elif webserver_enable_ssl_value == 'True':
		airflow_webserver_site_config_copy['base_url'] = "https://{0}:{1}".format(webserver_hostname_value,webserver_port_value)


	airflow_database_site_config = params.config['configurations']['airflow-database-site']
	airflow_username_value = airflow_database_site_config.get('airflow_username', '')
	airflow_password_value = airflow_database_site_config.get('airflow_password', '')
	airflow_database_name_value = airflow_database_site_config.get('airflow_database_name', '')
	hostname_value = airflow_database_site_config.get('hostname', '')
	database_type_value = airflow_database_site_config.get('database_type', '')

	celery_broker_value = airflow_database_site_config.get('celery_broker', '')
	rabbitmq_username_value = airflow_database_site_config.get('rabbitmq_username', '')
	rabbitmq_password_value = airflow_database_site_config.get('rabbitmq_password', '')
	rabbitmq_vhost_value = airflow_database_site_config.get('rabbitmq_vhost', '')


	airflow_core_site_config = params.config['configurations']['airflow-core-site'].copy()
	airflow_celery_site_config = params.config['configurations']['airflow-celery-site'].copy()

	if celery_broker_value == 'postgresql':
		airflow_celery_site_config['broker_url'] = "sqla+postgresql://{0}:{1}@{2}:5432/{3}".format(airflow_username_value, airflow_password_value, hostname_value, airflow_database_name_value)
	elif celery_broker_value == 'mysql':
		airflow_celery_site_config['broker_url'] = "sqla+mysql+pymysql://{0}:{1}@{2}:3306/{3}".format(airflow_username_value, airflow_password_value, hostname_value, airflow_database_name_value)
	elif celery_broker_value == 'rabbitmq':
		airflow_celery_site_config['broker_url'] = "pyamqp://{0}:{1}@{2}/{3}".format(rabbitmq_username_value, rabbitmq_password_value, hostname_value, rabbitmq_vhost_value)

	if database_type_value == 'postgresql':
		airflow_celery_site_config['result_backend'] = "db+postgresql://{0}:{1}@{2}:5432/{3}".format(airflow_username_value, airflow_password_value, hostname_value, airflow_database_name_value)
	elif database_type_value == 'mysql':
		airflow_celery_site_config['result_backend'] = "db+mysql+pymysql://{0}:{1}@{2}:3306/{3}".format(airflow_username_value, airflow_password_value, hostname_value, airflow_database_name_value)


	if database_type_value == 'postgresql':
		airflow_core_site_config['sql_alchemy_conn'] = "postgresql+psycopg2://{0}:{1}@{2}:5432/{3}".format(airflow_username_value, airflow_password_value, hostname_value, airflow_database_name_value)
	elif database_type_value == 'mysql':
		airflow_core_site_config['sql_alchemy_conn'] = "mysql+pymysql://{0}:{1}@{2}:3306/{3}".format(airflow_username_value, airflow_password_value, hostname_value, airflow_database_name_value)

	airflow_core_site_config['dags_folder'] = "{}{}".format(params.airflow_home, '/dags')

	#Kerberos
	airflow_kerberos_site_config = params.config['configurations']['airflow-kerberos-site'].copy()

	if not params.kerberos_disable and params.kerberos_security_enabled:
		airflow_kerberos_site_config['kinit_path'] = params.kinit_path_local
		airflow_make_systemd_scripts_kerberos(env)
		airflow_core_site_config['security'] = "kerberos"
	elif params.kerberos_disable and params.kerberos_security_enabled:
		airflow_core_site_config['security'] = ""

	airflow_config_file = ""

	airflow_config = airflow_generate_config_for_section({
		"core": airflow_core_site_config,
		"cli" : params.config['configurations']['airflow-cli-site'],
		"api" : params.config['configurations']['airflow-api-site'],
		"operators" : params.config['configurations']['airflow-operators-site'],
		"webserver" : airflow_webserver_site_config_copy,
		"email" : params.config['configurations']['airflow-email-site'],
		"smtp" : params.config['configurations']['airflow-smtp-site'],
		"celery" : airflow_celery_site_config,
		"dask" : params.config['configurations']['airflow-dask-site'],
		"scheduler" : params.config['configurations']['airflow-scheduler-site'],
		"ldap" : params.config['configurations']['airflow-ldap-site'],
		"mesos" : params.config['configurations']['airflow-mesos-site'],
		"kerberos" : airflow_kerberos_site_config,
		"github_enterprise" : params.config['configurations']['airflow-githubenterprise-site'],
		"admin" : params.config['configurations']['airflow-admin-site'],
		"lineage" : params.config['configurations']['airflow-lineage-site'],
		"atlas" : params.config['configurations']['airflow-atlas-site'],
		"hive" : params.config['configurations']['airflow-hive-site'],
		"elasticsearch" : params.config['configurations']['airflow-elasticsearch-site'],
		"kubernetes" : params.config['configurations']['airflow-kubernetes-site'],
		"kubernetes_secrets" : params.config['configurations']['airflow-kubernetessecrets-site'],
		"database" : params.config['configurations']['airflow-database-site'],
		"logging" : params.config['configurations']['airflow-logging-site']
		
	})

	for section, value in airflow_config.items():
		airflow_config_file += format("[{section}]\n{value}\n")

	with open(params.airflow_home + "/airflow.cfg", 'w') as configFile:
		configFile.write(airflow_config_file)
	configFile.close()