#!/usr/bin/python3
import sys, os, pwd, grp, signal, time
from resource_management import *
from subprocess import call
from airflow_setup import *
import subprocess

class AirflowWebserver(Script):
    """
    Contains the interface definitions for methods like install, 
    start, stop, status, etc. for the Airflow Server
    """
    def install(self, env):
        import params
        env.set_params(params)
        Logger.info(format("Installing Airflow Service"))
        Execute(format("python3 -m pip install --upgrade {airflow_pip_params} pip"), ignore_failures=True)
        Execute(format("python3 -m pip install --upgrade {airflow_pip_params} setuptools"), ignore_failures=True)
        Execute(format("python3 -m pip install --upgrade {airflow_pip_params} docutils pytest-runner Cython"), ignore_failures=True)
        self.install_packages(env)
        Execute(format("chmod 755 /usr/local/bin/airflow"))
        Execute(format("useradd {airflow_user}"), ignore_failures=True)
        Execute(format("mkdir -p {airflow_home}"))
        if not os.path.exists(params.base_log_folder):
            Execute(format("mkdir -p {base_log_folder}"))
            Execute(format("chown -R {airflow_user}:{airflow_group} {base_log_folder}"))
        airflow_make_startup_script(env)
        airflow_make_systemd_scripts_webserver(env)
        Execute(format("mkdir -p {airflow_home}/dags"))
        Execute(format("chown -R {airflow_user}:{airflow_group} {airflow_home}"))
        Execute(format("chmod 755 {airflow_home}"))
        Execute(format("chmod 755 {base_log_folder}"))
        Execute(format("chown -R {airflow_user}:{airflow_group} /var/run/airflow"))
        Execute(format("chmod 755 /var/run/airflow"))
        self.configure(env)
    
    def configure(self, env):
        import params
        env.set_params(params)
        airflow_configure(env)
        airflow_make_systemd_scripts_webserver(env)
        
    def start(self, env):
        import params
        self.configure(env)
        
        # Set ownership and permissions for /var/run/airflow
        Execute("sudo chown -R {0}:{1} /var/run/airflow".format(params.airflow_user, params.airflow_group))
        Execute("sudo chmod 755 /var/run/airflow")

        # Set ownership and permissions for base_log_folder
        Execute("sudo chown -R {0}:{1} {2}".format(params.airflow_user, params.airflow_group, params.base_log_folder))
        Execute("sudo chmod 755 {0}".format(params.base_log_folder))

        # Export AIRFLOW_HOME and start the Airflow webserver
        Execute(format("sudo systemctl daemon-reload"))
        Execute(format("sudo systemctl start airflow-webserver"))
        Execute(format("sudo systemctl enable airflow-webserver")) 
        
        # Capture the process ID (PID) of the Airflow webserver and write it to a new PID file
        Execute('ps -ef | grep "airflow-webserver" | grep -v grep | awk \'{print $2}\' | tail -n 1 > ' + params.airflow_webserver_pid_file, user=params.airflow_user)

        # Set ownership and permissions for the webserver PID file
        Execute("sudo chown {0}:{1} {2}".format(params.airflow_user, params.airflow_group, params.airflow_webserver_pid_file))
        Execute("sudo chmod 644 {}".format(params.airflow_webserver_pid_file))

    def stop(self, env):
        import params
        env.set_params(params)
        Execute(format("chown -R {airflow_user}:{airflow_group} /var/run/airflow"))
        Execute(format("chmod 755 /var/run/airflow"))

        Execute(format("sudo systemctl stop airflow-webserver"))
        Execute(format("sudo systemctl disable airflow-webserver")) 
        
        Execute(format("while sudo systemctl is-active airflow-webserver; do sleep 1; done"))

        # Delete the Airflow webserver PID file
        File(params.airflow_webserver_pid_file, action="delete")


    def restart(self, env):
        self.stop(env)
        time.sleep(5)
        self.start(env)

    def status(self, env):
        import status_params
        env.set_params(status_params)
        check_process_status(status_params.airflow_webserver_pid_file)

    def initdb(self, env):
        import params
        env.set_params(params)
        self.configure(env)
        Execute(format("export AIRFLOW_HOME={airflow_home} && AIRFLOW_CONFIG={airflow_home}/airflow.cfg && airflow db init"))
        Execute(format("export AIRFLOW_HOME={airflow_home} && AIRFLOW_CONFIG={airflow_home}/airflow.cfg && airflow users create --role Admin --username airflow --email airflow@airflow.io --firstname airflow --lastname User --password airflow"))

if __name__  == "__main__":
    AirflowWebserver().execute()
